function [emX, emY, time, trials] = eogmatrix (c, k)

% @CORTEX/EOGMATRIX   eog (eye) XY positions from a @cortex object
%
% [POSX,POSY,TIME,TRIALS] = EOGMATRIX(COBJ,KEY);
%
% where
% COBJ:   the @cortex object
% KEY:    a single key from a @key object
% POSX:   horizontal projections of eye positions;
% POSY:   vertical   projections of eye positions;
%         each matrix has COL samples (in dva) by ROW trials;
%         values are time-locked on the cue(s) specified in the key
% TIME:   the corresponding time scale
% TRIALS: a vector of trials actually included in POSX & POSY
%
% Last modified: 22 Dic 04

% based on eogmatrix.m, last saved on 19 May 00
% 24 May 00: help text revised
% 28 Jan 04: @key support added
% 23 Jun 04: help text fixed, included_trials output arg added
% 24 Jun 04: some improvements as in eppmatrix
% 28 Oct 04: cosmetics
%  5 Nov 04: bugfixes, help text, c.eog.XXX introduced
% 22 Dec 04: bug fixed: e.dva replaces c.eog2dva; more minor edits

error (NOTsinglekey (k));
yestrials = trialset (c, k);
sampling = unique (c.eog_rate(yestrials));
if length (sampling) > 1
   error ('variable sampling rate for EOG data is not supported');
end
e = c.eog;

samples_per_row = fix ((k.finish-k.start)/sampling);
emX = zeros (length (yestrials), samples_per_row);
emY = zeros (length (yestrials), samples_per_row);
instance = 0;
trials  = [];
cueless = [];
winless = [];
toomanycues = [];

for trial=yestrials
   codes = c.codes(trial, :);
   times = c.times(trial, :);
   % find the first and the last element of the vector 'e.X'
   % that contains the samples for the current trial
   % remember that c.eog_start (trial) == 0 if
   % the current trial does not contain any eog values
   first = e.ptr_start (trial);
   last  = e.ptr_end   (trial);
   samples = last-first+1;
   
   cues = ismember (codes, k.cues);
   switch sum(cues)
      case 0
         cueless = [cueless trial];
         continue;
      case 1
         cuetime = times (cues);
      otherwise
         toomanycues = [toomanycues trial];
         cuetime = times (min (find (cues)));
   end
   start_time = cuetime + k.start;
   end_time   = cuetime + k.finish;
   start_from_first = start_time - e.start(trial);
   if (start_from_first < 0 | (end_time > (start_time + samples * sampling)))
      winless = [winless trial];
   else
      instance = instance + 1;
      trials = [trials; trial]; 
      findx = first + fix (start_from_first/sampling);
      lindx = findx + samples_per_row-1;
      emX (instance, :) = double(e.X (findx:lindx))'/e.dva;
      emY (instance, :) = double(e.Y (findx:lindx))'/e.dva;
   end
end
emX = emX (1:instance, :);
emY = emY (1:instance, :);
time = k.start:sampling:k.finish-sampling;

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
if ~isempty (winless)
   warning ('could not retrieve all requested trials;');
   disp ('time window exceeds boundaries of the following trial(s)');
   disp (winless);
end
if ~isempty (toomanycues)
   warning ('more than one cue event found in the trials listed below;');
   disp ('only the first one in each trial was considered');
   disp (toomanycues)
end
