function [dense, inst] = trialdensity (c, cluster, window, type, r, group)
% trialdensity.m
% returns trial-by-trial spike density arrays
%
% Syntax 1: [DENSE, INSTANCES] = TRIALDENSITY (COBJ, CLUSTER, WINDOW, TYPE, ROBJ, [GROUP])
% Syntax 2: [DENSE, INSTANCES] = TRIALDENSITY (SPMAT, WINDOW, TYPE)
% Syntax 3: [DENSE, INSTANCES] = TRIALDENSITY (FRATES, WINDOW, TYPE);
%
% where
% DENSE:     array of spike density values;
% INSTANCES: number of spike arrays retrieved from COBJ by the RULES criteria
%            i.e., the number of values included in the sum;
% COBJ:      the @cortex object;
% CLUSTER:   the spike number, as encoded in the CORTEX data file;
% SPMAT:     a spike matrix as returned by @cortex/spikematrix.m;
% FRATES:    a 1D vector of values to smooth;
% WINDOW:    window size in ms;
% TYPE:      either 'gauss' or 'boxcar';
% ROBJ:      one rule from a @rules object 
%            or the entire object, if GROUP is specified;
% GROUP:     the group to be analyzed (optional).
%
% based on density.m (last modified 2 Jun 00)
% Last modified: 2 Jun 00

switch nargin
case 6
   spmat = spikematrix (c, cluster, r, group);
case 5
   if sum(r.size)>1
      error ('multi-rule object passed using single-rule syntax');
   end
   spmat = spikematrix (c, cluster, r);
case 3
   spmat = c;
   type = window;
   window = cluster;
   if ndims(spmat)>2 | prod (size(window))>1 | ~isstr(type)
      error ('bad argument list');
   end
otherwise
   error ('wrong number of arguments');
end

% make sure the window is of even size
window = window - rem (window, 2);
halfwin = window/2;
[inst, spl] = size (spmat);
if inst==0, dense=[]; return, end;
switch type
case 'gauss'
   x = -halfwin:halfwin;
   kernel = exp(-x.^2/halfwin^2);
case 'boxcar'
   kernel = ones (window + 1, 1);
otherwise
   error ('type must be either ''gauss'' or ''boxcar''');
end
kernel = kernel / sum (kernel);
padded = zeros (inst, spl + window * 2);
padded (:, halfwin + 1 : spl + halfwin) = spmat;
padded (:, 1:halfwin) = repmat (mean(spmat(:,1:halfwin), 2), 1, halfwin);
padded (:, halfwin + spl + 1 : spl + window) = repmat (mean (spmat (:, spl - halfwin : spl), 2), 1, halfwin);
dense = filter (kernel, 1, padded')';
dense = dense (:, window+1 : end-window) * 1000;