function roc_area = rocgiu(list1, list2, plot_it, mode)

% rocgiu.m
%
% Takes two lists (e.g., trial-by-trial firing rates in
% two experimental conditions) and computes ROC area for them.
% First list should be the one corresponding to "noise".
% In comparing stimulus evoked response of a cell vs. spontaneous 
% firing rate, SER should be entered as list2.
%
% Syntax: ROC_AREA = ROC(NOISE, SIGNAL, PLOT_IT, MODE)
%
% Where:
%   PLOT_IT: pass a non-zero value to create a figure
%   MODE should be one of:
%        'keep':  process lists as passed (roc_area < .5 allowed)
%        'sort':  force signal to list with higher mean (roc_area > .5)
%        'min50': process lists as passed, but force roc_area >= .5
%
% John Reynolds 7/30/98
% Giuseppe: 8/27/99
% Giuseppe: 16 april 2001 --> fixed a few things...

% Normally, first list is assumed to be "signal" and second is assumed to be "noise".
% If signal_is_higher is set = 1, then the list with higher mean is treated as signal, 
% which makes it unlikely to get ROC value < 0.5

min_50 = 0;

if prod (size (list1)) ~= length (list1)
   error ('list1 is not a 1D vector')
end
if prod (size (list2)) ~= length (list2)
   error ('list2 is not a 1D vector')
end

switch mode
case 'sort'
   if mean(list1) > mean(list2) % Make list2 the one with bigger mean
      temp  = list2;
      list2 = list1;
      list1 = temp;
   end
case 'min50'
   min_50 = 1;
end

size1 = length(list1);
size2 = length(list2);

list1 = sort(list1);
list2 = sort(list2);
hival = max(list1(end), list2(end));

list1 (end+1 : end + max(1, 1+size2-size1)) = hival;
list2 (end+1 : end + max(1, 1+size1-size2)) = hival;

rpt1 = [0:1/size1:1 1];
rpt2 = [0:1/size2:1 1];

idx1 = 0;	% The index of the last element in list1 that matched the tested value.
idx2 = 0;	% The index of the last element in list1 that matched the tested value.

% Go through the sorted lists, creating a list of points on the ROC curve.
curve 	= [0 0]; % This will become the list of points in the ROC curve
tv = min(list1(1), list2(1)) - 1; % Start checking values just below lowest observed value.
while tv < hival
   nt1 = list1 (idx1+1);
   nt2 = list2 (idx2+1);
   tv = min (nt1, nt2);
   if nt1 == tv, idx1 = idx1+1; end
   if nt2 == tv, idx2 = idx2+1; end
   curve = [curve; [rpt2(idx2+1), rpt1(idx1+1)]];
end
curve (end, :) = [1 1];

% Compute the area under the ROC curve.
roc_area = sum( ...
   (curve(2:end,1)-curve(1:end-1,1)).* ...
   (curve(2:end,2)+curve(1:end-1,2))./2);

if min_50 == 1
   roc_area = max(roc_area, 0.5);   
end

% If a plot is desired, provide it.
if plot_it
   figure; 
   plot(curve(:,1),curve(:,2));
   axis square;
   grid on;
   set (gca, ...
      'xlim', [0 1], ...
      'ylim', [0 1], ...
      'XTick',[0:.25:1], ...
      'YTick',[0:.25:1])
   xlabel('P(value>threshold),list with smaller mean');
   ylabel('P(value>threshold),list with larger mean');
   pagetext(.5, .95, ['ROC Area = ' num2str(roc_area)]);
end
